/*
   check read/write behaviour with/without buffer for pipe.c
   this is becaus i/o could be very slow if used in a poor way
   
   fwrite/fread => w/r-blocks      Linux-2.4 SunOS-5.8 IRIX-6.5  Tru64    Jun03
   fread(b,1,20k,f)               4k+16k/20k     8k/8k  20k/20k  20k/20k
   fread(b,20k,1,f)               4k+16k/20k     8k/8k  20k/20k  20k/20k
   fread(b,1,20k,f) vbuf=0           20k/20k    20k/1   20k/20k  20k/1   Sun,Tru64=slow (speed/40)
   fread(b,20k,1,f) vbuf=0           20k/20k    20k/1   20k/20k  20k/1
   fread(b,1,20k,f) vbuf=1M         200k/1M    200k/1M  20k/20k 200k/1M
   fread(b,20k,1,f) vbuf=1M         200k/1M    200k/1M  20k/20k 200k/1M
                                                        max200k?
   pipe via dd bs=...                                    bs
   ToDo: man unlocked_stdio (thread-unsafe) (BSD/GNU) + read/write (unistd)
     open(...,O_LARGEFILE+O_NOATIME+O_DIRECT?
     altix-SLES9 read/write 256kB blocks?
   
   gcc -O2 -o vbuf vbuf.c                            
   Linux: strace -f -F -r -T -s 5 vbuf
   SunOS: truss -f -a -l -D -d vbuf
   IRIX:  par -i -A -a 5 -d -u  vbuf
   Tru64: ./trace -f -T -P -b 5 -s 5 ../a.out # Alpha-trace 1.34 Jan99

*/
#include <stdlib.h>
#include <stdio.h>

#define BSIZE (1024*1024)  /* big buffer */
#define DSIZE (  20*1024)  /* handy data */

char buff[BSIZE];
char data[DSIZE];

int main(int argn, char *argv[]) {
  FILE *f1;
  int i;
  for(i=0;i<DSIZE;i++) data[i]=(char)i;  /* initialize data */

  /* write using default buffer */
  f1=fopen("/dev/null","wb");
  for(i=0;i<10;i++) fwrite(data,1,DSIZE,f1); /* repeat 100 times? */
  fclose(f1);
  f1=fopen("/dev/null","wb");
  for(i=0;i<10;i++) fwrite(data,DSIZE,1,f1); /* repeat 100 times? */
  fclose(f1);
 
  /* no buffer */
  f1=fopen("/dev/null","wb");
  setvbuf(f1,(char *)0,_IONBF,0);
  for(i=0;i<10;i++) fwrite(data,1,DSIZE,f1); /* repeat 100 times? */
  fclose(f1);
  f1=fopen("/dev/null","wb");
  setvbuf(f1,(char *)0,_IONBF,0);
  for(i=0;i<10;i++) fwrite(data,DSIZE,1,f1); /* repeat 100 times? */
  fclose(f1);
  
  /* big buffer */
  f1=fopen("/dev/null","wb");
  setvbuf(f1,buff,_IOFBF,BSIZE);
  for(i=0;i<10;i++) fwrite(data,1,DSIZE,f1); /* repeat 100 times? */
  fclose(f1);
  f1=fopen("/dev/null","wb");
  setvbuf(f1,buff,_IOFBF,BSIZE);
  for(i=0;i<10;i++) fwrite(data,DSIZE,1,f1); /* repeat 100 times? */
  fclose(f1);

  /* write using default buffer */
  f1=fopen("/dev/zero","rb");
  for(i=0;i<10;i++) fread(data,1,DSIZE,f1); /* repeat 100 times? */
  fclose(f1);
  f1=fopen("/dev/zero","rb");
  for(i=0;i<10;i++) fread(data,DSIZE,1,f1); /* repeat 100 times? */
  fclose(f1);
 
  /* no buffer */
  f1=fopen("/dev/zero","rb");
  setvbuf(f1,(char *)0,_IONBF,0);
  for(i=0;i<10;i++) fread(data,1,DSIZE,f1); /* repeat 100 times? */
  fclose(f1);
  f1=fopen("/dev/zero","rb");
  setvbuf(f1,(char *)0,_IONBF,0);
  for(i=0;i<10;i++) fread(data,DSIZE,1,f1); /* repeat 100 times? */
  fclose(f1);
  
  /* big buffer */
  f1=fopen("/dev/zero","rb");
  setvbuf(f1,buff,_IOFBF,BSIZE);
  for(i=0;i<10;i++) fread(data,1,DSIZE,f1); /* repeat 100 times? */
  fclose(f1);
  f1=fopen("/dev/zero","rb");
  setvbuf(f1,buff,_IOFBF,BSIZE);
  for(i=0;i<10;i++) fread(data,DSIZE,1,f1); /* repeat 100 times? */
  fclose(f1);
}
